/*
 * s-rcon.c
 *
 * written and (c) 2002 by Sean Finney
 *
 * This program can be freely copied and redistributed under
 * the terms of the GNU General Public License, v2.  For details
 * of said terms, please see the file COPYING
 *
 */

//	Includes
#include "common.h"
#include "netio.h"
#include "session.h"

//	Definitions

//	Macros

//	Function Prototypes
void usage(void);
void version(void);
void process_args(int ac, char *av[]);
void bail(char *reason, int exitval);

//	Global (config) Variables 

struct option program_opts[] = {
	{ "help",0,0,'h' },
	{ "version",0,0,'v' },

	{ "command",1,0,'c' },
	{ "no-rcon",0,0,'n' },
	{ "outfile",0,0,'o' },
	{ "port",1,0,'p' },
	{ "password",1,0,'P'},
	{ "server",1,0,'s' },
	{ NULL, 0, 0, 0}
};

extern char *optarg;

global_config_t g_config;

//	---- MAIN ----
int main(int argc, char *argv[]){
	FILE *conn_file;
	long sess_id;
	process_args(argc, argv);

	conn_file=netio_connect(g_config.server, g_config.port);
	if(!g_config.no_rcon){
		sess_id=session_rcon_establish(conn_file);
		if(sess_id==-1) bail("unable to establish a session id", 1);
		else fprintf(stderr, "established session (id %ld)\n", sess_id);
		if(g_config.cmd) 
			session_rcon_cmd(conn_file, sess_id, g_config.cmd);
	} else {
		if(g_config.cmd) 
			session_client_cmd(conn_file, g_config.cmd);
	}


	fclose(conn_file);
	return 0;
}

//	---- Other Functions ----

void process_args(int ac, char *av[]){
	int c, portno;
	short merge_stderr=0;
	char *outfn=NULL;

	memset(&g_config, 0, sizeof(g_config));
	g_config.progname=basename(av[0]);

	while(1){
		c=getopt_long(ac, av, "c:hmno:p:P:s:v", program_opts, NULL);
		switch(c){
		case 'c': g_config.cmd=strdup(optarg); break;
		case 'h': version(); usage(); exit(0); break;
		case 'm': merge_stderr=1; break;
		case 'n': g_config.no_rcon=1; break;
		case 'o': outfn=strdup(optarg); break;
		case 'p': 
			  portno=atoi(optarg);
			  if(portno <= 0) bail("port number what?", 1);
			  else g_config.port=portno;
			  break;
		case 'P':
			  g_config.password=strdup(optarg);
			  break;
		case 's': g_config.server=strdup(optarg); break;
		case 'v': version(); exit(0); break;
		} 
		if(c==-1)break;
	}

	if(!outfn) g_config.outfile=stdout;
	else g_config.outfile=fopen(outfn, "w");
	if(!g_config.outfile) {
		perror("opening output stream");
		bail("can't write output", 1);
	}
	if(merge_stderr) g_config.errfile=g_config.outfile;
	else g_config.errfile=stderr;

	if(!g_config.server) g_config.server=strdup("localhost");
	if(!g_config.port) g_config.port=27015;
}

void version(void){
	fprintf(stderr, "%s v%s\t-\thalf-life remote console utility\n", 
			g_config.progname, PACKAGE_VERSION);
	fprintf(stderr, "copyright 2002 sean finney <seanius@seanius.net>\n");
}

void usage(void){
	fprintf(stderr, "\nusage:\t");
	fprintf(stderr, "%s [OPTIONS] ...\n", g_config.progname);
	fprintf(stderr, 
"  -h, --help                    this informative usage summary :)\n"
"  -v, --version                 the current version and copyright\n\n"
"  -c, --command=COMMAND         issue COMMAND to server\n"
"  -n, --no-rcon                 send command as a standard client request\n"
"  -p, --port=PORT               connect on port PORT\n"
"  -P, --password=STRING         use STRING as password\n"
"  -s, --server=SERVER           use SERVER as the ip/hostname of the server\n"
	       );
}

void bail(char *reason, int exitval){
	if(reason) fprintf(stderr, "%s: bailing: %s\n", 
			g_config.progname, reason);
	exit(exitval);
}
